use actix_web::{
    body::MessageBody,
    dev::{ServiceRequest, ServiceResponse},
    http::header::{HeaderValue, CONTENT_SECURITY_POLICY},
    Error, HttpMessage,
};
use actix_web_lab::middleware::Next;
use rand::Rng;
use std::fmt::Display;

pub struct ScriptNonce(String);

impl Display for ScriptNonce {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.0)
    }
}

pub async fn insert_script_nonce(
    req: ServiceRequest,
    next: Next<impl MessageBody>,
) -> Result<ServiceResponse<impl MessageBody>, Error> {
    let script_nonce = format!("{:02x}", rand::thread_rng().gen::<u128>());
    req.extensions_mut()
        .insert(ScriptNonce(script_nonce.clone()));
    let mut res = next.call(req).await;
    if let Ok(res) = res.as_mut() {
        let value = format!("default-src 'none'; connect-src 'self'; img-src 'self'; media-src 'self'; font-src 'self'; script-src 'nonce-{script_nonce}'; style-src 'self'; object-src 'none'; base-uri 'self'; frame-src 'none'; frame-ancestors 'none'; form-action 'self'; require-trusted-types-for 'script';");
        res.headers_mut().insert(
            CONTENT_SECURITY_POLICY,
            HeaderValue::from_str(&value).unwrap(),
        );
    }
    res
}
